package com.supervision.knowsub.service.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.supervision.knowsub.domain.UserInfo;
import com.supervision.knowsub.entity.vo.homepage.MyKnowledgeStatisticsResVO;
import com.supervision.knowsub.enums.RoleTypeEnum;
import com.supervision.knowsub.enums.StatusEnum;
import com.supervision.knowsub.exception.BusinessException;
import com.supervision.knowsub.model.Knowledge;
import com.supervision.knowsub.model.KnowledgeFlowTodo;
import com.supervision.knowsub.model.SystemUser;
import com.supervision.knowsub.service.*;
import com.supervision.knowsub.util.UserUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Slf4j
@Service
@RequiredArgsConstructor
public class HomepageServiceImpl implements HomepageService {

    private final KnowledgeService knowledgeService;

    private final SystemUserService systemUserService;

    private final KnowledgeFlowTodoService knowledgeFlowTodoService;

    private final KnowledgeFlowRecordService knowledgeFlowRecordService;

    private final KnowledgeSubmitRecordService knowledgeSubmitRecordService;

    /**
     * 先看用户是否是知识审批角色
     * 如果是知识审批角色,则按知识审批角色统计
     * 如没有知识审批角色，则以报送人员视角进行统计
     * 如没有任何角色权限，数据全部显示为0
     * <p>
     * 知识数量:如果多次审批,则进行去重,只保留一次
     * <p>
     * 报送人员视角：
     * 待审批：我提交的且当前处于“待审批”状态的知识数量。
     * 审批通过：我提交的且被标记为“审批通过”的知识数量。
     * 驳回：我提交的且被标记为“驳回”的知识数量。
     * 失效: 我提交的且被标记为“失效”的知识数量。
     * 全量报送：我提交的所有知识数量（无论状态如何）。
     * 报送人员的知识: 只要被我发起过的,即使别人给再次发起,也算我的知识,这是也应该被统计
     * <p>
     * 知识审批人员视角：
     * 待审批：待我审批的知识数量。
     * 审批通过：由我审批并标记为“审批通过”的知识数量。
     * 驳回：由我审批并标记为“驳回”的知识数量。
     * 失效: 由我审批过的知识,并标记为“失效”的知识数量。
     * 全量报送：所有审批通过的知识数量（包括我和其他审批者审批通过的）
     *
     * @return 页面展示的结果
     */
    @Override
    public MyKnowledgeStatisticsResVO queryMyKnowledgeStatistics() {
        UserInfo user = UserUtil.getUser();

        SystemUser systemUser = systemUserService.getOptById(user.getId()).orElseThrow(() -> new BusinessException("未找到用户"));
        MyKnowledgeStatisticsResVO resVO = new MyKnowledgeStatisticsResVO();
        resVO.setUserName(user.getUsername());
        resVO.setLastLoginTime(systemUser.getRecentLoginTime());
        // 首先查询用户是否有审批的权限,
        List<Integer> roleTypeList = user.getRoleTypeList();
        log.info("用户角色信息:{}", JSONUtil.toJsonStr(roleTypeList));
        Set<Integer> passStatusSet = new HashSet<>(List.of(new Integer[]{2, 11, 12, 16, 17}));
        Long passTotalCount = knowledgeService.lambdaQuery().in(Knowledge::getStatus, passStatusSet).count();
        resVO.setKnowledgeCount(passTotalCount);
        // 如果用户有审批权限,则以审批人的视角
        if (roleTypeList.contains(RoleTypeEnum.KNOWLEDGE_APPROVAL.getRoleType())) {
            log.info("用户有知识审批权限,以审批人的视角进行统计");
            // 待审批
            resVO.setWaitProcessCount(knowledgeFlowTodoService.lambdaQuery().eq(KnowledgeFlowTodo::getTodoUserId, user.getId()).count());
            // 由我审批并标记为审批通过的数量
            resVO.setPassProcessCount(knowledgeFlowRecordService.queryPassProcessCountByUser(user.getId(), 1, null));
            // 由我审批并标记为“驳回”的知识数量。
            resVO.setRejectProcessCount(knowledgeFlowRecordService.queryPassProcessCountByUser(user.getId(), 2, null));
            // 查询由我审批且目前已失效的
            resVO.setInvalidProcessCount(knowledgeFlowRecordService.queryPassProcessCountByUser(user.getId(), null, 5));
        }
        // 没有被配置审批权限，则以报送人员视角进行统计
        else if (roleTypeList.contains(RoleTypeEnum.KNOWLEDGE_SUBMIT.getRoleType())) {
            log.info("用户没有被配置知识审批权限，则以报送人员视角进行统计");
            // 待审批：我提交的且当前处于“待审批”状态的知识数量。
            resVO.setWaitProcessCount(knowledgeSubmitRecordService.queryKnowledgeSubmitTodoCount(user.getId()));
            // 驳回的数量
            Long recallSubmitCount = knowledgeSubmitRecordService.queryKnowledgeSubmitStatusCount(user.getId(), StatusEnum.RECALL_APPROVAL_SUBMIT.getStatus());
            Long recallDeleteCount = knowledgeSubmitRecordService.queryKnowledgeSubmitStatusCount(user.getId(), StatusEnum.RECALL_APPROVAL_DELETE.getStatus());
            Long recallRecallCount = knowledgeSubmitRecordService.queryKnowledgeSubmitStatusCount(user.getId(), StatusEnum.RECALL_APPROVAL_RECALL.getStatus());
            resVO.setRejectProcessCount(NumberUtil.add(recallSubmitCount, recallDeleteCount, recallRecallCount).longValue());
            resVO.setInvalidProcessCount(knowledgeSubmitRecordService.queryKnowledgeSubmitStatusCount(user.getId(), StatusEnum.INVALID.getStatus()));
            resVO.setPassProcessCount(knowledgeSubmitRecordService.queryKnowledgeSubmitStatusCount(user.getId(), StatusEnum.PASS.getStatus()));
        } else {
            log.info("用户没有任何角色权限，就只展示全量的,其他的都是0");
        }
        // 没有任何角色权限，就只展示全量的,其他的都是0
        return resVO;
    }
}
