package com.supervision.knowsub.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.domain.UserInfo;
import com.supervision.knowsub.dto.base.UserBaseOperateAuthDTO;
import com.supervision.knowsub.entity.vo.knowledge.*;
import com.supervision.knowsub.enums.FlowTypeEnum;
import com.supervision.knowsub.enums.StatusEnum;
import com.supervision.knowsub.exception.BusinessException;
import com.supervision.knowsub.model.*;
import com.supervision.knowsub.service.*;
import com.supervision.knowsub.util.UserUtil;
import com.supervision.knowsub.vo.knowledge.KnowledgePageResVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class KnowledgeManageServiceImpl implements KnowledgeManageService {

    private final KnowledgeService knowledgeService;

    private final KnowledgeContextService knowledgeContextService;

    private final KnowledgeInfoService knowledgeInfoService;

    private final KnowledgeAttachmentService knowledgeAttachmentService;

    private final KnowledgeLinkService knowledgeLinkService;

    private final KnowledgeFlowRecordService flowRecordService;

    private final FileService fileService;

    private final KnowledgeFlowService knowledgeFlowService;

    private final SystemDeptService systemDeptService;

    private final SystemBaseService systemBaseService;


    @Override
    public IPage<KnowledgePageResVO> queryKnowledgePage(Integer status,
                                                        String title,
                                                        String publishDeptId,
                                                        String baseId,
                                                        String submitDeptId,
                                                        LocalDate publishDateBegin,
                                                        LocalDate publishDateEnd, LocalDate submitDateBegin,
                                                        LocalDate submitDateEnd, Integer pageNum, Integer pageSize) {
        String userId = UserUtil.getUser().getId();
        // 除全部可以看所有外,其他tab都是看到自己报送的知识的状态(或参与过的)
        IPage<KnowledgePageResVO> knowledgeIPage = knowledgeService.queryKnowledgePage(status, title, publishDeptId, baseId, submitDeptId, publishDateBegin, publishDateEnd, submitDateBegin, submitDateEnd, userId, pageNum, pageSize);
        List<KnowledgePageResVO> records = knowledgeIPage.getRecords();
        if (CollUtil.isEmpty(records)) {
            return knowledgeIPage;
        }
        // 获取所有部门
        Set<String> submitDeptSet = records.stream().map(Knowledge::getSubmitDeptId).collect(Collectors.toSet());
        Set<String> publishDeptSet = records.stream().map(Knowledge::getPublishDeptId).collect(Collectors.toSet());
        Collection<String> deptIdSet = CollUtil.union(submitDeptSet, publishDeptSet);
        Map<String, String> deptMap = systemDeptService.listByIds(deptIdSet).stream().collect(Collectors.toMap(SystemDept::getId, SystemDept::getDeptName));
        // 获取所有子库
        Set<String> baseIdSet = records.stream().map(Knowledge::getBaseId).collect(Collectors.toSet());
        Map<String, String> baseMap = systemBaseService.listByIds(baseIdSet).stream().collect(Collectors.toMap(SystemBase::getId, SystemBase::getBaseName));
        List<KnowledgePageResVO> list = records.stream().map(knowledge -> {
            KnowledgePageResVO node = BeanUtil.copyProperties(knowledge, KnowledgePageResVO.class);
            node.setBaseName(baseMap.getOrDefault(knowledge.getBaseId(), "全部"));
            node.setSubmitDeptName(deptMap.getOrDefault(knowledge.getSubmitDeptId(), "未知部门"));
            node.setPublishDeptName(deptMap.getOrDefault(knowledge.getPublishDeptId(), "未知部门"));
            return node;
        }).toList();
        // 进行只是的标记操作
        markKnowledgeOperationFlag(list);
        // status是查全部,则在页面展示的时候,需要把16,17进行转换
        if (ObjectUtil.isNull(status)) {
            for (KnowledgePageResVO knowledgePageResVO : list) {
                // 如果状态是16,17,则在页面展示的时候,展示为通过
                if (Objects.equals(knowledgePageResVO.getStatus(), StatusEnum.RECALL_APPROVAL_RECALL.getStatus())
                        || Objects.equals(knowledgePageResVO.getStatus(), StatusEnum.RECALL_APPROVAL_DELETE.getStatus())) {
                    knowledgePageResVO.setStatus(StatusEnum.PASS.getStatus());
                }
            }
        }
        knowledgeIPage.setRecords(list);
        return knowledgeIPage;
    }

    /**
     * 标记知识的操作状态
     */
    private void markKnowledgeOperationFlag(List<KnowledgePageResVO> list) {
        // 获取用户所有子库的操作权限
        UserInfo user = UserUtil.getUser();
        List<UserBaseOperateAuthDTO> baseOperateAuthDTOList = systemBaseService.queryUserBaseOperateAuth(user.getId());
        Map<String, Set<Integer>> baseOperateAuthMap = baseOperateAuthDTOList.stream()
                .collect(Collectors.groupingBy(UserBaseOperateAuthDTO::getBaseId, Collectors.mapping(UserBaseOperateAuthDTO::getFlowType, Collectors.toSet())));
        // 首先需要看用户对这个子库的权限是什么,如果用户有该子库的对应的操作权限,则允许进行对应操作
        for (KnowledgePageResVO knowledgePageResVO : list) {
            String baseId = knowledgePageResVO.getBaseId();

            if (StatusEnum.DRAFT.getStatus().equals(knowledgePageResVO.getStatus())) {
                // 如果是草稿,就只允许编辑和删除
                knowledgePageResVO.setModifyFlag(1);
                knowledgePageResVO.setDeleteFlag(1);
            } else if (StatusEnum.PASS.getStatus().equals(knowledgePageResVO.getStatus())) {
                // 如果是已发布,就只允许撤回.需要有该子库权限
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.RECALL.getFlowType())) {
                    knowledgePageResVO.setRecallFlag(1);
                }
                // 已发布的,也可以直接删除,但是也要校验权限
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.DELETE.getFlowType())) {
                    knowledgePageResVO.setDeleteFlag(1);
                }

            } else if (StatusEnum.RECALL_APPROVAL_SUBMIT.getStatus().equals(knowledgePageResVO.getStatus())) {
                // 如果是新增驳回,就可以支持编辑和删除,但是需要校验是否需要有权限
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.SUBMIT.getFlowType())) {
                    knowledgePageResVO.setModifyFlag(1);
                }
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.DELETE.getFlowType())) {
                    knowledgePageResVO.setDeleteFlag(1);
                }
            } else if (StatusEnum.RECALL_APPROVAL_DELETE.getStatus().equals(knowledgePageResVO.getStatus())
                    || StatusEnum.RECALL_APPROVAL_RECALL.getStatus().equals(knowledgePageResVO.getStatus())) {
                // 如果是删除驳回和撤回驳回,就可以继续支持撤回和删除流程,但是需要校验是否需要有权限
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.SUBMIT.getFlowType())) {
                    knowledgePageResVO.setRecallFlag(1);
                }
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.DELETE.getFlowType())) {
                    knowledgePageResVO.setDeleteFlag(1);
                }
            } else if (StatusEnum.INVALID.getStatus().equals(knowledgePageResVO.getStatus())) {
                // 如果是已失效,支持编辑撤回删除,但是需要校验是否有对应的权限
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.SUBMIT.getFlowType())) {
                    knowledgePageResVO.setModifyFlag(1);
                }
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.RECALL.getFlowType())) {
                    knowledgePageResVO.setRecallFlag(1);
                }
                if (baseOperateAuthMap.containsKey(baseId) && baseOperateAuthMap.get(baseId).contains(FlowTypeEnum.DELETE.getFlowType())) {
                    knowledgePageResVO.setDeleteFlag(1);
                }
            }
            // 其他状态就全部置灰
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveKnowledge(SaveKnowledgeReqVO reqVO) {
        if (2 == reqVO.getOperate()) {
            Assert.notBlank(reqVO.getBaseId(), "应用子库不能为空");
            KnowledgeInfo knowledgeInfo = reqVO.getKnowledgeInfo();
            Assert.notNull(knowledgeInfo, "有效日期不能为空");
            Assert.notBlank(reqVO.getPublishDeptId(), "发文部门不能为空");
            Assert.notNull(knowledgeInfo.getPublishDate(), "发文时间不能为空");
            Assert.notNull(knowledgeInfo.getTimeliness(), "有效日期不能为空");
        }
        var context = new KnowledgeContext();
        context.setContext(reqVO.getContent());
        knowledgeContextService.save(context);
        // 保存基本信息
        KnowledgeInfo baseInfo = Optional.ofNullable(reqVO.getKnowledgeInfo()).orElse(new KnowledgeInfo());
        baseInfo.setContextId(context.getId());
        knowledgeInfoService.save(baseInfo);

        Knowledge knowledge = new Knowledge();

        knowledge.setTitle(reqVO.getTitle());
        knowledge.setContentId(context.getId());
        knowledge.setInfoId(baseInfo.getId());
        knowledge.setBaseId(reqVO.getBaseId());
        knowledge.setPublishDeptId(reqVO.getPublishDeptId());
        knowledge.setExcerpt(reqVO.getExcerpt());
        // 目前只有人工添加
        knowledge.setKnowledgeFrom(1);
        knowledge.setStatus(reqVO.getOperate() == 1 ? StatusEnum.DRAFT.getStatus() : StatusEnum.WAIT_APPROVAL.getStatus());
        knowledge.setSubmitDeptId(UserUtil.getUser().getDeptId());
        if (1 == reqVO.getOperate()) {
            knowledge.setDraftBelongUserId(UserUtil.getUser().getId());
        }
        knowledgeService.save(knowledge);
        // 保存知识的基本信息

        // 附件以及URL没添加
        for (String fileId : reqVO.getFileIdList()) {
            KnowledgeAttachment attachment = new KnowledgeAttachment();
            attachment.setKnowledgeId(knowledge.getId());
            attachment.setFileId(fileId);
            knowledgeAttachmentService.save(attachment);
        }
        for (KnowledgeLinkVO knowLedgeLinkVO : reqVO.getLinkList()) {
            KnowledgeLink knowledgeLink = new KnowledgeLink();
            knowledgeLink.setKnowledgeId(knowledge.getId());
            knowledgeLink.setLinkName(knowLedgeLinkVO.getLinkName());
            knowledgeLink.setLinkUrl(knowLedgeLinkVO.getLinkUrl());
            knowledgeLinkService.save(knowledgeLink);
        }
        // 这里需要进行判断,如果状态为2,说明要走流程
        if (1 != reqVO.getOperate()) {
            // 这里需要走流程
            UserInfo user = UserUtil.getUser();
            knowledgeFlowService.submitFlowProcess(knowledge, FlowTypeEnum.SUBMIT, user.getId(), user.getDeptId(), reqVO.getRemark());
            // 如果提交了审批,就把草稿所属人置为空
            knowledgeService.lambdaUpdate().set(Knowledge::getDraftBelongUserId, null).eq(Knowledge::getId, knowledge.getId()).update();
        }


    }

    /**
     * @param reqVO
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateKnowledge(ModifyKnowledgeReqVO reqVO) {
        if (2 == reqVO.getOperate()) {
            Assert.notBlank(reqVO.getBaseId(), "应用子库不能为空");
            KnowledgeInfo knowledgeInfo = reqVO.getKnowledgeInfo();
            Assert.notNull(knowledgeInfo, "有效日期不能为空");
            Assert.notBlank(reqVO.getPublishDeptId(), "发文部门不能为空");
            Assert.notNull(knowledgeInfo.getPublishDate(), "发文时间不能为空");
            Assert.notNull(knowledgeInfo.getTimeliness(), "有效日期不能为空");
        }

        Knowledge knowledge = knowledgeService.getOptById(reqVO.getKnowledgeId()).orElseThrow(() -> new BusinessException("未找到知识"));
        // 首先校验状态,草稿/已撤回/已失效/驳回,才能修改(通过的,要先撤回才能修改)
        Set<Integer> canModifyStatusSet = Set.of(StatusEnum.DRAFT.getStatus(), StatusEnum.RECALL.getStatus(), StatusEnum.INVALID.getStatus(), StatusEnum.RECALL_APPROVAL_SUBMIT.getStatus());
        if (!canModifyStatusSet.contains(knowledge.getStatus())) {
            throw new BusinessException("当前状态不支持修改");
        }
        // 找到对应的content
        KnowledgeContext context = knowledgeContextService.getOptById(knowledge.getContentId()).orElse(KnowledgeContext.builder().id(reqVO.getKnowledgeId()).build());
        context.setContext(reqVO.getContent());
        context.updateById();

        // 然后更新
        knowledge.setTitle(reqVO.getTitle());
        knowledge.setExcerpt(reqVO.getExcerpt());
        knowledge.setBaseId(reqVO.getBaseId());
        knowledge.setPublishDeptId(reqVO.getPublishDeptId());
        knowledge.setSubmitDeptId(UserUtil.getUser().getDeptId());
        knowledge.setStatus(reqVO.getOperate() == 1 ? StatusEnum.DRAFT.getStatus() : StatusEnum.WAIT_APPROVAL.getStatus());

        // 如果保存为草稿
        if (1 == reqVO.getOperate()) {
            knowledge.setDraftBelongUserId(UserUtil.getUser().getId());
        }
        knowledgeService.updateById(knowledge);
        // 更新基础信息
        KnowledgeInfo knowledgeInfo = reqVO.getKnowledgeInfo();
        // 这里需要用SQL进行更新,不能用封装的updateById,因为前端某些字段可能变为空,用updateById不能让那个想为空的变成空
        knowledgeInfoService.lambdaUpdate()
                .set(KnowledgeInfo::getPolicyType, knowledgeInfo.getPolicyType())
                .set(KnowledgeInfo::getTerritory, knowledgeInfo.getTerritory())
                .set(KnowledgeInfo::getPublishScope, knowledgeInfo.getPublishScope())
                .set(KnowledgeInfo::getKnowledgeTag, knowledgeInfo.getKnowledgeTag())
                .set(KnowledgeInfo::getExecTimeBegin, knowledgeInfo.getExecTimeBegin())
                .set(KnowledgeInfo::getExecTimeEnd, knowledgeInfo.getExecTimeEnd())
                .set(KnowledgeInfo::getTimeliness, knowledgeInfo.getTimeliness())
                .set(KnowledgeInfo::getPublishDate, knowledgeInfo.getPublishDate())
                .eq(KnowledgeInfo::getId, knowledgeInfo.getId()).update();
        // 有可能更新附件
        updateKnowledgeFile(reqVO, knowledge);
        // 有可能更新URL
        updateKnowledgeLink(reqVO, knowledge);
        // 这里需要进行判断,如果状态为2,说明要走流程
        if (1 != reqVO.getOperate()) {
            UserInfo user = UserUtil.getUser();
            knowledgeFlowService.submitFlowProcess(knowledge, FlowTypeEnum.SUBMIT, user.getId(), user.getDeptId(), reqVO.getRemark());
            // 将状态修改问待审批
            knowledgeService.lambdaUpdate().set(Knowledge::getStatus, StatusEnum.WAIT_APPROVAL.getStatus()).eq(Knowledge::getId, knowledge.getId()).update();
            // 如果提交了审批,就把草稿所属人置为空
            knowledgeService.lambdaUpdate().set(Knowledge::getDraftBelongUserId, null).eq(Knowledge::getId, knowledge.getId()).update();

        }
    }

    private void updateKnowledgeLink(ModifyKnowledgeReqVO reqVO, Knowledge knowledge) {
        // 直接删除原来的
        knowledgeLinkService.lambdaUpdate().eq(KnowledgeLink::getKnowledgeId, knowledge.getId()).remove();
        for (KnowledgeLinkVO knowLedgeLinkVO : reqVO.getLinkList()) {
            KnowledgeLink knowledgeLink = new KnowledgeLink();
            knowledgeLink.setKnowledgeId(knowledge.getId());
            knowledgeLink.setLinkName(knowLedgeLinkVO.getLinkName());
            knowledgeLink.setLinkUrl(knowLedgeLinkVO.getLinkUrl());
            knowledgeLinkService.save(knowledgeLink);
        }

    }

    private void updateKnowledgeFile(ModifyKnowledgeReqVO reqVO, Knowledge knowledge) {
        // 获取数据库中的附件ID
        List<KnowledgeAttachment> oldFileList = knowledgeAttachmentService.lambdaQuery().eq(KnowledgeAttachment::getKnowledgeId, knowledge.getId()).list();
        if (CollUtil.isEmpty(reqVO.getFileIdList())) {
            if (CollUtil.isNotEmpty(oldFileList)) {
                knowledgeAttachmentService.lambdaUpdate().eq(KnowledgeAttachment::getKnowledgeId, knowledge.getId()).remove();
                // 删除文件
                for (KnowledgeAttachment knowledgeAttachment : oldFileList) {
                    fileService.deleteFileById(knowledgeAttachment.getFileId());
                }
            }

        } else {
            List<String> newFileIdList = reqVO.getFileIdList();
            // 遍历旧的附件,如果新的附件列表中不包含,则删除
            for (KnowledgeAttachment oldFile : oldFileList) {
                if (!newFileIdList.contains(oldFile.getId())) {
                    knowledgeAttachmentService.lambdaUpdate().eq(KnowledgeAttachment::getFileId, oldFile.getId()).remove();
                    fileService.deleteFileById(oldFile.getFileId());
                }
            }
            List<String> oldFileIdList = oldFileList.stream().map(KnowledgeAttachment::getFileId).toList();
            // 再添加新的附件
            for (String fileId : reqVO.getFileIdList()) {
                if (!oldFileIdList.contains(fileId)) {
                    KnowledgeAttachment attachment = new KnowledgeAttachment();
                    attachment.setKnowledgeId(knowledge.getId());
                    attachment.setFileId(fileId);
                    knowledgeAttachmentService.save(attachment);
                }

            }
        }

    }

    @Override
    public void recallKnowledge(String knowledgeId, String remark) {
        Knowledge knowledge = knowledgeService.getOptById(knowledgeId).orElseThrow(() -> new BusinessException("未找到知识"));
        // 只有通过状态或已失效状态才支持撤回
        Set<Integer> canRecallStatusSet = Set.of(StatusEnum.PASS.getStatus(), StatusEnum.INVALID.getStatus());
        if (!canRecallStatusSet.contains(knowledge.getStatus())) {
            throw new BusinessException("该知识状态不能撤回");
        }
        // 走撤回流程
        UserInfo user = UserUtil.getUser();
        knowledgeFlowService.submitFlowProcess(knowledge, FlowTypeEnum.RECALL, user.getId(), user.getDeptId(), remark);
        // 然后将状态修改问带审核
        knowledgeService.lambdaUpdate().set(Knowledge::getStatus, StatusEnum.WAIT_APPROVAL_RECALL.getStatus()).eq(Knowledge::getId, knowledge.getId()).update();
    }

    @Override
    public KnowledgeDetailResVO queryKnowledgeDetail(String knowledgeId) {
        Knowledge knowledge = knowledgeService.getOptById(knowledgeId).orElseThrow(() -> new BusinessException("未找到知识"));
        KnowledgeInfo knowledgeInfo = knowledgeInfoService.getOptById(knowledge.getInfoId()).orElse(new KnowledgeInfo());
        KnowledgeContext knowledgeContext = knowledgeContextService.getOptById(knowledge.getContentId()).orElse(new KnowledgeContext());
        List<KnowledgeLink> knowledgeLinkList = knowledgeLinkService.lambdaQuery().eq(KnowledgeLink::getKnowledgeId, knowledgeId).list();
        List<KnowledgeAttachment> knowledgeAttachmentList = knowledgeAttachmentService.lambdaQuery().eq(KnowledgeAttachment::getKnowledgeId, knowledgeId).list();
        List<String> fileIdList = knowledgeAttachmentList.stream().map(KnowledgeAttachment::getFileId).toList();
        List<FileInfo> fileInfos = fileService.listByFileId(fileIdList);
        // 查看详情的时候,需要返回相应的库名和部门名称
        KnowledgeDetailResVO knowledgeDetailResVO = new KnowledgeDetailResVO(knowledge, knowledgeInfo, knowledgeContext, knowledgeLinkList, fileInfos);
        knowledgeDetailResVO.setBaseName(systemBaseService.getOptById(knowledge.getBaseId()).map(SystemBase::getBaseName).orElse("未知子库"));
        knowledgeDetailResVO.setSubmittedDeptName(systemDeptService.getOptById(knowledge.getSubmitDeptId()).map(SystemDept::getDeptName).orElse("未知部门"));
        knowledgeDetailResVO.setPublishDept(systemDeptService.getOptById(knowledge.getPublishDeptId()).map(SystemDept::getDeptName).orElse("未知部门"));
        return knowledgeDetailResVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteKnowledge(String knowledgeId, String remark) {
        // 删除是逻辑删除,如果当前状态是草稿,且没有走过审批流程,则可以直接删除
        Knowledge knowledge = knowledgeService.getOptById(knowledgeId).orElseThrow(() -> new BusinessException("未找到知识"));
        // 如果非草稿,则校验是否是通过/驳回/已撤回/已失效状态,如果是,则需要走审批流程进行删除
        Set<Integer> canDeleteStatusSet = Set.of(StatusEnum.PASS.getStatus(), StatusEnum.RECALL_APPROVAL_RECALL.getStatus(), StatusEnum.RECALL_APPROVAL_DELETE.getStatus(), StatusEnum.INVALID.getStatus());
        // 如果是审批草稿,需要进行判断之前是否有过记录
        if (Objects.equals(StatusEnum.DRAFT.getStatus(), knowledge.getStatus())) {
            // 需要判断之前是否走过审批流程
            if (flowRecordService.lambdaQuery().eq(KnowledgeFlowRecord::getKnowledgeId, knowledgeId).count() == 0) {
                // 如果没有走过审批的记录,则可以直接删除
                knowledgeService.removeById(knowledgeId);
                knowledgeInfoService.removeById(knowledge.getInfoId());
                knowledgeContextService.lambdaUpdate().eq(KnowledgeContext::getId, knowledge.getContentId()).remove();
                // 删除文件
                List<KnowledgeAttachment> list = knowledgeAttachmentService.lambdaQuery().eq(KnowledgeAttachment::getKnowledgeId, knowledgeId).list();
                List<String> fileIdList = list.stream().map(KnowledgeAttachment::getFileId).toList();
                fileService.listByFileId(fileIdList);
                knowledgeAttachmentService.lambdaUpdate().eq(KnowledgeAttachment::getKnowledgeId, knowledgeId).remove();
            } else {
                // 如果有过审批记录,则直接修改状态为逻辑删除
                knowledgeService.lambdaUpdate().set(Knowledge::getStatus, StatusEnum.DELETE.getStatus())
                        .set(Knowledge::getDraftBelongUserId, null)
                        .eq(Knowledge::getId, knowledge.getId()).update();
            }
        } else if (Objects.equals(StatusEnum.RECALL_APPROVAL_SUBMIT.getStatus(), knowledge.getStatus())) {
            // 如果是新增驳回的,需要判断除了本次驳回的,是否有被审批通过过,如果有被报送且审批通过过,就要走流程进行删除
            if (flowRecordService.lambdaQuery().eq(KnowledgeFlowRecord::getKnowledgeId, knowledgeId).eq(KnowledgeFlowRecord::getFlowType, FlowTypeEnum.SUBMIT.getFlowType()).eq(KnowledgeFlowRecord::getProcessStatus, 1).count() > 0) {
                // 如果有,则走流程进行删除
                // 这时需要走审批流程进行删除
                UserInfo user = UserUtil.getUser();
                knowledgeFlowService.submitFlowProcess(knowledge, FlowTypeEnum.DELETE, user.getId(), user.getDeptId(), remark);
                // 然后将状态修改为待审核
                knowledgeService.lambdaUpdate().set(Knowledge::getStatus, StatusEnum.WAIT_APPROVAL_DELETE.getStatus()).eq(Knowledge::getId, knowledge.getId()).update();
            }else {
                // 如果有过审批记录,则直接修改状态为逻辑删除
                knowledgeService.lambdaUpdate().set(Knowledge::getStatus, StatusEnum.DELETE.getStatus())
                        .set(Knowledge::getDraftBelongUserId, null)
                        .eq(Knowledge::getId, knowledge.getId()).update();
            }
        } else if (canDeleteStatusSet.contains(knowledge.getStatus())) {
            // 这时需要走审批流程进行删除
            UserInfo user = UserUtil.getUser();
            knowledgeFlowService.submitFlowProcess(knowledge, FlowTypeEnum.DELETE, user.getId(), user.getDeptId(), remark);
            // 然后将状态修改为待审核
            knowledgeService.lambdaUpdate().set(Knowledge::getStatus, StatusEnum.WAIT_APPROVAL_DELETE.getStatus()).eq(Knowledge::getId, knowledge.getId()).update();
        } else {
            throw new BusinessException("审批中的知识不支持删除");
        }


    }


}
